"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _constants = require("../constants");

var _importResource = require("../import-resource");

const importMiddlewares = ({
  resolveConfig,
  isClient
}) => {
  var _resolveConfig$middle;

  if (isClient) {
    throw new Error(`${_constants.message.serverAliasInClientCodeError}$resolve.middlewares`);
  }

  const imports = [`import '$resolve.guardOnlyServer'`];
  const constants = [];
  const exports = [``, `const middlewares = { command: [], resolver: [], projection: [] }`, ``];
  const {
    aggregate: commandMiddlewares = [],
    readModel: {
      resolver: resolverMiddlewares = [],
      projection: projectionMiddlewares = []
    } = {}
  } = (_resolveConfig$middle = resolveConfig.middlewares) !== null && _resolveConfig$middle !== void 0 ? _resolveConfig$middle : {};

  for (let index = 0; index < commandMiddlewares.length; index++) {
    (0, _importResource.importResource)({
      resourceName: `command_middlewares_${index}`,
      resourceValue: commandMiddlewares[index],
      runtimeMode: _constants.RUNTIME_ENV_OPTIONS_ONLY,
      importMode: _constants.RESOURCE_ANY,
      instanceMode: _constants.IMPORT_INSTANCE,
      imports,
      constants
    });
    exports.push(`middlewares.command.push(command_middlewares_${index})`);
  }

  for (let index = 0; index < resolverMiddlewares.length; index++) {
    (0, _importResource.importResource)({
      resourceName: `resolver_middlewares_${index}`,
      resourceValue: resolverMiddlewares[index],
      runtimeMode: _constants.RUNTIME_ENV_OPTIONS_ONLY,
      importMode: _constants.RESOURCE_ANY,
      instanceMode: _constants.IMPORT_INSTANCE,
      imports,
      constants
    });
    exports.push(`middlewares.resolver.push(resolver_middlewares_${index})`);
  }

  for (let index = 0; index < projectionMiddlewares.length; index++) {
    (0, _importResource.importResource)({
      resourceName: `projection_middlewares_${index}`,
      resourceValue: projectionMiddlewares[index],
      runtimeMode: _constants.RUNTIME_ENV_OPTIONS_ONLY,
      importMode: _constants.RESOURCE_ANY,
      instanceMode: _constants.IMPORT_INSTANCE,
      imports,
      constants
    });
    exports.push(`middlewares.projection.push(projection_middlewares_${index})`);
  }

  exports.push(`export default middlewares`);
  return [...imports, ...constants, ...exports].join('\r\n');
};

var _default = importMiddlewares;
exports.default = _default;
//# sourceMappingURL=$resolve.middlewares.js.map