"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _loaderUtils = _interopRequireDefault(require("loader-utils"));

var _path = _interopRequireDefault(require("path"));

var _resolveResource = require("../resolve-resource");

var _constants = require("../constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const importReadModelProcedure = ({
  resolveConfig,
  isClient
}, resourceQuery) => {
  if (!/^\?/.test(resourceQuery)) {
    throw new Error(`Resource $resolve.readModelProcedure should be retrieved with resource query`);
  }

  if (isClient) {
    throw new Error(`${_constants.message.serverAliasInClientCodeError}$resolve.readModelProcedure`);
  }

  const {
    readModelName
  } = _loaderUtils.default.parseQuery(resourceQuery);

  const readModel = resolveConfig.readModels.find(({
    name
  }) => name === readModelName);
  const readModelConnector = readModel != null ? resolveConfig.readModelConnectors[readModel.connectorName] : null;
  let wrapProcedureMethodPath = null;

  for (const pathPostfix of [['es', 'wrap-procedure'], ['lib', 'wrap-procedure'], ['wrap-procedure']]) {
    try {
      const {
        result
      } = (0, _resolveResource.resolveResource)(_path.default.join(readModelConnector.module, ...pathPostfix));
      wrapProcedureMethodPath = result;
    } catch (e) {}

    if (wrapProcedureMethodPath != null) {
      break;
    }
  }

  if (wrapProcedureMethodPath == null) {
    throw new Error(_constants.OPTIONAL_ASSET_ERROR);
  }

  const imports = [`import { SynchronousPromise } from 'synchronous-promise'`, `import currentReadModel from '$resolve.readModel?readModelName=${readModelName}&onlyProjection=true'`, `import wrapProcedure from ${JSON.stringify(wrapProcedureMethodPath)}`];
  const constants = [`SynchronousPromise.installGlobally()`, `const procedure = wrapProcedure(currentReadModel)`];
  const exports = [`export default procedure`];
  return [...imports, ...constants, ...exports].join('\r\n');
};

var _default = importReadModelProcedure;
exports.default = _default;
//# sourceMappingURL=$resolve.readModelProcedure.js.map