"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getResetDomainConfig = exports.default = void 0;

var _merge = _interopRequireDefault(require("./merge"));

var _generate_custom_mode = _interopRequireDefault(require("./generate_custom_mode"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const validOptions = ['dropEventStore', 'dropReadModels', 'dropEventSubscriber', 'dropSagas', 'bootstrap'];

const getResetDomainConfig = async (resolveConfig, options) => {
  if (options == null || options.constructor !== Object) {
    throw new Error('Invalid reset options');
  }

  for (const key of Object.keys(options)) {
    if (!validOptions.includes(key) || options[key] == null || options[key].constructor !== Boolean) {
      throw new Error(`Invalid reset options: ${key}`);
    }
  }

  const config = (0, _merge.default)(resolveConfig, {
    apiHandlers: [{
      method: 'GET',
      path: '/api/reset-domain',
      handler: {
        module: {
          package: '@resolve-js/runtime-single-process',
          import: 'resetDomainHandler'
        },
        options
      }
    }, {
      method: 'OPTIONS',
      path: '/SKIP_COMMANDS',
      handler: {
        package: '@resolve-js/runtime-base',
        import: 'failHandler'
      }
    }]
  });
  return config;
};

exports.getResetDomainConfig = getResetDomainConfig;
const reset = (0, _generate_custom_mode.default)(getResetDomainConfig, 'reset-domain');
var _default = reset;
exports.default = _default;
//# sourceMappingURL=reset_mode.js.map